# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: install.py 358 2008-07-29 18:56:21Z flarumbe $
#
# end: Platecom header
"""
$Id: install.py 358 2008-07-29 18:56:21Z flarumbe $
"""
from StringIO import StringIO

from Products.CMFCore.utils import getToolByName
from Products.Archetypes.utils import shasattr
from Products.Archetypes.Extensions.utils import install_subskin
from icsemantic.thesaurus.config import *
from icsemantic.thesaurus.interfaces import IThesaurus
from icsemantic.thesaurus.Thesaurus import Thesaurus

def install_configlets(out):
    """
    Method to install platecom user properties...
        @type: portal: PloneSite
        @type out: StringIO

        @rtype: StringIO
        @return: Messages from the GS process
    """
    configTool=getToolByName(portal(self),'portal_controlpanel',None)
    if configTool:
        for conf in CONFIGLETS:
            configTool.registerConfiglet(**conf)
            out.write('Added configlet %s\n' % conf['id'])

    return out

def install_dependencies( self ):
    """
    Test if everything at DEPENDENCIES gets installed...
        >>> 1
        1
    """
    # If the config contains a list of dependencies, try to install
    # them.  Add a list called DEPENDENCIES to your custom
    # AppConfig.py (imported by config.py) to use it.
    out = StringIO()
    quickinstaller = portal(self).portal_quickinstaller
    for dependency in DEPENDENCIES:
        print >> out, "Installing dependency %s:" % dependency
        quickinstaller.installProduct(dependency)

    return out

def install( self ):
    """
    External module to install the product...
        @type self: PloneSite
        @param self: The Plone site object

        @rtype: StringIO
        @return: Messages from the install process

    some tests here...
        >>> from icsemantic.thesaurus.config import *
        >>> qi = self.portal.portal_quickinstaller
        >>> installed = [ prod['id'] for prod in qi.listInstalledProducts() ]
        >>> PACKAGENAME in installed
        True

    """
    
    out = StringIO()
    print >> out, "Installing Dependencies"
    install_dependencies(self)

    setup_tool = getToolByName(self, 'portal_setup')
    if shasattr(setup_tool, 'runAllImportStepsFromProfile'):
        # Plone 3
        pass
    else:
        # Plone 2.5.  Would work on 3.0 too, but then it gives tons of
        # DeprecationWarnings when running the tests, causing failures
        # to drown in the noise.
        pass

    install_thesaurus_utility(self, out)

    install_subskin(self, out, GLOBALS)

    res = import_gs_profiles(self, out)

    return out.getvalue()

def install_thesaurus_utility(self, out):
    portal = getToolByName(self, 'portal_url').getPortalObject()
    sm = portal.getSiteManager()
    if not sm.queryUtility(IThesaurus, name=''):
	try:
            sm.registerUtility(Thesaurus(),IThesaurus,'')
	except:
            sm.registerUtility(IThesaurus,Thesaurus(),'')
    
    out.write("Adding Thesaurus Utility\n")

def addDocument(portal, document_id, document_title, keywords):
    portal.invokeFactory("Document", document_id)
    doc = portal[document_id]
    doc.setTitle(document_title)
    doc.setSubject(keywords)
    return doc

def deleteDocument(portal, document_id):
    portal.manage_delObjects(document_id)

def import_gs_profiles(self, out):
    """
    Method to install GS profiles...
        @type out: StringIO
        @param out: The object to append the output

        @rtype: StringIO
        @return: Messages from the GS process

    some tests here...
        >>> from icsemantic.core.config import *
        >>> psetup = self.portal.portal_setup

    just test we have registered the profile...
        >>> profilename = PROJECTNAME + ':default'
        >>> PACKAGENAME in [profile['product'] for profile in psetup.listProfileInfo()]
        True
        >>> profilename in [profile['id'] for profile in psetup.listProfileInfo()]
        True

    now we can test some stuff modified but that template...
        >>> 'icSemantic' in [ai.getTitle() for ai in portal.portal_actionicons.listActionIcons()]
        True

    No se porque este no anda, anda bien en el test funcional...
        >>> # [ai['name'] for ai in portal.portal_controlpanel.listActionInfos()] True


    """
    # Run all import steps
    print >> out, "Import GS Profiles"
    setup_tool = getToolByName(portal(self), 'portal_setup')
    profile_name = 'profile-' + PROJECTNAME + ':default'
    if shasattr(setup_tool, 'runAllImportStepsFromProfile'):
        # Plone 3
        print >> out, setup_tool.runAllImportStepsFromProfile(profile_name)
    else:
        # Plone 2.5.  Would work on 3.0 too, but then it gives tons of
        # DeprecationWarnings when running the tests, causing failures
        # to drown in the noise.
        old_context = setup_tool.getImportContextID()
        print >> out, setup_tool.setImportContext(profile_name)
        print >> out, setup_tool.runAllImportSteps()
        print >> out, setup_tool.setImportContext(old_context)

    return out

def uninstall( self ):
    """
    Test if everything gets installed...
        >>> 1
        1
    """
    out = StringIO()

    print >> out, "Uninstalling"

    return out.getvalue()

def portal(self):
    return getToolByName(self, 'portal_url').getPortalObject()



# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: setuphandlers.py 265 2008-06-13 05:20:36Z crocha $
#
# end: Platecom header
""" CMFDefault setup handlers.

"""
from StringIO import StringIO

from zope.interface import alsoProvides, directlyProvides, directlyProvidedBy
from zope.component import getUtility
from zope.app.component.hooks import setSite
from zope.app.component.interfaces import ISite, IPossibleSite
from Products.CMFCore.utils import getToolByName
from Products.Five.site.localsite import enableLocalSiteHook

from icsemantic.core.interfaces import IicSemanticSite
from icsemantic.thesaurus import add_keywords_incrementalsearchwidget_property_ifnotexists, keywords_incrementalsearchwidget
from icsemantic.thesaurus.replaceKeywordsWidget import replace_keywords_widget_at_metadata_page
import interfaces
from preferences import icSemanticThesaurusConfiguration
from config import HAS_PLONE3

def ensure_site(context):
  """Ensure the given context implements ISite.  The importance of
  this method is that it will ensure the given context is an ISite
  regardless of the Zope version (Zope 2.9 had a really hacked up
  SiteManager mechanism we have to account for).

      >>> from zope.app.component.interfaces import ISite, IPossibleSite
      >>> from OFS.Folder import Folder
      >>> if not IPossibleSite.implementedBy(Folder):
      ...    from zope import interface
      ...    from Products.Five.site.metaconfigure import (FiveSite,
      ...                                                  classSiteHook)
      ...    classSiteHook(Folder, FiveSite)
      ...    interface.classImplements(Folder, IPossibleSite)
      >>> om = Folder('foo')

      >>> ISite.providedBy(om)
      False

      >>> from icsemantic.core.setuphandlers import ensure_site
      >>> ensure_site(om)
      >>> ISite.providedBy(om)
      True

  """
  if not IPossibleSite.providedBy(context):
      if hasattr(context, 'getPhysicalPath'):
          p = '/'.join(context.getPhysicalPath())
      elif hasattr(context, 'getId'):
          p = context.getId()
      elif hasattr(context, 'id'):
          p = id
      else:
          p = str(context)

      raise TypeError('The object, "%s", is not an IPossibleSite' % p)

  if not ISite.providedBy(context):
    enableLocalSiteHook(context)
    setSite(context)

  if not ISite.providedBy(context):
    raise TypeError('Somehow trying to configure "%s" as an ISite '
                    'has failed' % '/'.join(context.getPhysicalPath()))

def setup_site(portal, out):
    """

        >>> from icsemantic.thesaurus import interfaces
        >>> from zope.app.component.hooks import setSite
        >>> setSite(portal)

        >>> sm = portal.getSiteManager()
        >>> pmct = sm.queryUtility(interfaces.IicSemanticManagementContentTypes,
        ...                        name='icsemantic.configuration')
        >>> pmct # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
        <icSemanticManagementContentTypes at /plone.../>

        >>> pmct.fallback_types = []


    """
    alsoProvides(portal, IicSemanticSite)
    sm = portal.getSiteManager()

    if not sm.queryUtility(interfaces.IicSemanticThesaurusConfiguration, name='icsemantic.thesaurus.configuration'):
        if HAS_PLONE3:
            sm.registerUtility(icSemanticThesaurusConfiguration(),
                               interfaces.IicSemanticThesaurusConfiguration,
                               'icsemantic.thesaurus.configuration')
        else:
            sm.registerUtility(interfaces.IicSemanticThesaurusConfiguration,
                               icSemanticThesaurusConfiguration(),
                               'icsemantic.thesaurus.configuration')
    app = portal.restrictedTraverse('/')
    add_keywords_incrementalsearchwidget_property_ifnotexists(app)
    if keywords_incrementalsearchwidget(app):
        replace_keywords_widget_at_metadata_page(out)

def importVarious(context):
  """ Import various settings.

  This provisional handler will be removed again as soon as full handlers
  are implemented for these steps.
  """
  site = context.getSite()
  out = StringIO()
  logger = context.getLogger("icsemantic.thesaurus")

  ensure_site(site)
  setup_site(site, out)

  print >> out, 'Various settings imported.'

  logger.info(out.getvalue())
  return out.getvalue()

def unimportVarious(context):
  """ Import various settings.

  This provisional handler will be removed again as soon as full handlers
  are implemented for these steps.
  """
  site = context.getSite()
  out = StringIO()
  logger = context.getLogger("icsemantic.thesaurus")

  print >> out, 'Various settings unimported.'

  logger.info(out.getvalue())
  return out.getvalue()



# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: admin.py 262 2008-06-12 21:14:31Z flarumbe $
#
# end: Platecom header
"""
admin setting and preferences
Solo vistas y forms

@author: Juan Pablo Gimenez
@contact: jpg@rcom.com.ar
"""
__author__ = """Juan Pablo Gimenez <jpg@rcom.com.ar>"""
__docformat__ = 'plaintext'

import os
from datetime import datetime
from StringIO import StringIO

import zope
from zope import component
from zope.component import getUtility
from zope.formlib import form
from zope.app.form.browser import MultiSelectSetWidget
from zope.app.form.browser.itemswidgets import MultiSelectWidget \
        as BaseMultiSelectWidget, DropdownWidget, SelectWidget
from zope.app.form.browser import FileWidget
from zope.app.component.hooks import getSite

try:
    from zope.lifecycleevent import ObjectModifiedEvent
except:
    from zope.app.event.objectevent import ObjectModifiedEvent

from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
from Products.Five.browser import BrowserView
from Products.Five.formlib import formbase

from icsemantic.core import pkg_home
from icsemantic.core.i18n import _
from icsemantic.core.vocabularies import LanguagesVocabularyFactory
from icsemantic.core.browser.base import BaseSettingsForm
from icsemantic.thesaurus import interfaces
from icsemantic.thesaurus import update_keywords_incrementalsearchwidget_property
from icsemantic.thesaurus.replaceKeywordsWidget import replace_keywords_widget_at_metadata_page, replace_keywords_widget_to_original_at_metadata_page
from icsemantic.thesaurus.Thesaurus import thesaurus_utility
from icsemantic.thesaurus.browser.incrementalSearchSelectWidget import IncrementalSearchSelectWidgetFactory
from azaxview import AzaxView

class ThesaurusUpload( BaseSettingsForm ):
    """ Configlet for upload a thesaurus rdf file.
    """
    form_name = _(u'Thesaurus upload')
    form_fields = form.Fields( interfaces.IicSemanticManagementThesaurusUpload )
    form_fields['thesaurus_file'].custom_widget = FileWidget

    @form.action(_("Apply"), condition=form.haveInputWidgets)
    def handle_edit_action(self, action, data):
        if form.applyChanges(self.context,
                             self.form_fields,
                             data,
                             self.adapters):
            self.load_thesaurus(data['thesaurus_file'],
			    data['default_language'],
			    contexts=[data['thesaurus_context']],
			    format=data['thesaurus_format'],
			    encoding=data['encoding'],
			    new=data['new'])

            zope.event.notify(
                ObjectModifiedEvent(self.context)
                )
            self.status = _(
                "Updated on ${date_time}",
                mapping={'date_time': str(datetime.utcnow())}
                )
        else:
            self.status = _('No changes')

    def load_thesaurus(self, file_content, default_language,
		    contexts=[], format='SKOSCore', encoding='utf-8',
		    new=False):
        # TODO: add a field Language
        thesaurus_utility().load_string(file_content, default_language,
			contexts=contexts, format=format, encoding=encoding,
			new=new)

class ThesaurusDownload( BrowserView ):
    """ Download thesaurus and query concepts as an RDF File.
    """
    def rdf_string(self):
        return thesaurus_utility().rdf_string()

    def rdf_term_concepts(self):
        return thesaurus_utility().rdf_term_concepts(self.term_parameter(), self.context_parameter())

    def rdf_concepts_search(self):
        return thesaurus_utility().rdf_concepts_search(self.search_parameter(), self.context_parameter())

    def context_parameter(self):
        if self.request.has_key('context'):
            return self.request['context']
        else:
            return None

    def term_parameter(self):
        return self.request['term']

    def search_parameter(self):
        return self.request['search']

class ThesaurusQuery( BrowserView ):
    """ Query the thesaurus.
    """

    def query_result(self):
        return self.azaxview().query_result(self.paging_begin_index(), self.paging_end_index())
    
    def paging_header(self):
        atFirstPage = self.paging_begin_index() == 0
        atLastPage = self.paging_end_index() == len(self.query_result_terms())
        if atFirstPage and atLastPage:
            html = "All results"
        else:
            html = "Page: <select id=\"pageRange\" name=\"pageRange\">" + self.html_page_options() + "</select>"
        return html

    def newTermLanguage(self):
        return "<select id=\"newTermLanguage\" name=\"newTermLanguage\">" + self.html_newTermLanguage_options() + "</select>"
    
    def html_page_options(self):
        html = ""
        for page in range(1, self.last_page()+1):
            html += "<option value=\"%d:%d\"" % (self.paging_begin_index_of(page), self.paging_end_index_of(page))
            if page == self.page():
                html += " selected"
            html += ">%d</option>" % page
        return html
    
    def html_newTermLanguage_options(self):
        html = ""
        for language in LanguagesVocabularyFactory(self.context):
            html += "<option value=\"%s\"" % language.value
            if language.value == self.defaultLanguage():
                html += " selected"
            html += ">%s</option>" % language.title
        return html

    def defaultLanguage(self):
        return preferredLanguage()
    
    def paging_range_of(self, page):
        if self.listIsPaged():
            elementsbypage = self.elementsByPage()
            begin_index = (page - 1) * elementsbypage
            end_index = min(begin_index + elementsbypage, len(self.query_result_terms()))
        else:
            begin_index = 0
            end_index = len(self.query_result_terms())
        return (begin_index, end_index)

    def paging_range(self):
        return paging_range_of(self.page())

    def paging_begin_index(self):
        return self.paging_begin_index_of(self.page())
    
    def paging_end_index(self):
        return self.paging_end_index_of(self.page())

    def paging_begin_index_of(self, page):
        return self.paging_range_of(page)[0]
    
    def paging_end_index_of(self, page):
        return self.paging_range_of(page)[1]
    
    def last_page(self):
        if self.listIsPaged():
            page = (len(self.query_result_terms()) - 1) // self.elementsByPage() + 1
        else:
            page = 1
        return page

    def listIsPaged(self):
        return not self.has_parameter('showall')
    
    def elementsByPage(self):
        if self.has_parameter('elementsbypage'):
            return int(self.request['elementsbypage'])
        else:
            return 20
    
    def page(self):
        if self.has_parameter('page'):
            return int(self.request['page'])
        else:
            return 1

    def has_parameter(self, parameter):
        return self.request.has_key(parameter) and self.request[parameter] != ""
    
    def query_result_terms(self):
        return self.azaxview().query_result_terms()
    
    def azaxview(self):
        if not hasattr(self, "_azaxview"):
            self.setAzaxview(AzaxView(self.context, self.request))
        return self._azaxview
    
    def setAzaxview(self, azaxview):
        self._azaxview = azaxview


class VerticalSelectTest( BaseSettingsForm ):
    """ Test the incremental search select widget.
    """
    form_name = _(u'Vertical Select Test Form')
    form_fields = form.Fields( interfaces.IicSemanticVerticalSelectTest )
    form_fields['vertical_select'].custom_widget = IncrementalSearchSelectWidgetFactory

    @form.action(_("Apply"), condition=form.haveInputWidgets)
    def handle_edit_action(self, action, data):
        if form.applyChanges(self.context,
                             self.form_fields,
                             data,
                             self.adapters):
            zope.event.notify(
                ObjectModifiedEvent(self.context)
                )
            self.status = _(
                "Updated on ${date_time}.\nConcepts selected: ${selected_concepts}",
                mapping={
                    'date_time': str(datetime.utcnow()),
                    'selected_concepts': str(data['vertical_select'])
                }
                )
        else:
            self.status = _('No changes')

class icSemanticThesaurusConfigurationForm( BaseSettingsForm ):
	""" It is a configlet that allow to change thesaurus configuration.
	"""
	form_name = _(u'Thesaurus Configuration')
	form_fields = form.Fields( interfaces.IicSemanticThesaurusConfiguration )

	@form.action(_("Apply"), condition=form.haveInputWidgets)
	def handle_edit_action(self, action, data):
		if form.applyChanges(self.context,
							 self.form_fields,
							 data,
							 self.adapters):
			app = self.context.restrictedTraverse('/')
			update_keywords_incrementalsearchwidget_property(app, data['keywords_incrementalsearchwidget'])
			if data['keywords_incrementalsearchwidget']:
				replace_keywords_widget_at_metadata_page(StringIO())
			else:
				replace_keywords_widget_to_original_at_metadata_page(StringIO())

			zope.event.notify(
				ObjectModifiedEvent(self.context)
				)
			self.status = _(
				"Updated on ${date_time}",
				mapping={'date_time': str(datetime.utcnow())}
				)
		else:
			self.status = _('No changes')
    
def preferredLanguage():
    return portal_languages().getPreferredLanguage()

def supportedLanguages():
    return portal_languages().getSupportedLanguages()

def portal_languages():
    return getToolByName(getSite(), 'portal_languages')



# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: preferences.py 358 2008-07-29 18:56:21Z flarumbe $
#
# end: Platecom header
from zope.interface import implements
from zope.component import getUtility
from OFS.SimpleItem import SimpleItem
from zope.schema.fieldproperty import FieldProperty
from icsemantic.core.fieldproperty import ToolDependentFieldProperty
from icsemantic.thesaurus import exist_keywords_incrementalsearchwidget_property
from icsemantic.thesaurus import keywords_incrementalsearchwidget

import interfaces

class icSemanticManagementThesaurusUpload:
    implements(interfaces.IicSemanticManagementThesaurusUpload)

    def __init__(self, context):
        """
        """
        self.context = context

    def __call__(self):
        pass

    thesaurus_file = FieldProperty(interfaces.IicSemanticManagementThesaurusUpload['thesaurus_file'])
    default_language = FieldProperty(interfaces.IicSemanticManagementThesaurusUpload['default_language'])
    thesaurus_context = FieldProperty(interfaces.IicSemanticManagementThesaurusUpload['thesaurus_context'])
    thesaurus_format = FieldProperty(interfaces.IicSemanticManagementThesaurusUpload['thesaurus_format'])
    encoding = FieldProperty(interfaces.IicSemanticManagementThesaurusUpload['encoding'])
    new = FieldProperty(interfaces.IicSemanticManagementThesaurusUpload['new'])

class icSemanticVerticalSelectTest:
    implements(interfaces.IicSemanticVerticalSelectTest)

    def __init__(self, context):
        self.context = context

    def __call__(self):
        pass

    vertical_select = FieldProperty(interfaces.IicSemanticVerticalSelectTest['vertical_select'])

def thesaurus_upload_adapter(context):
    return icSemanticManagementThesaurusUpload(context)

def vertical_select_test_adapter(context):
    return icSemanticVerticalSelectTest(context)

class icSemanticThesaurusConfiguration(SimpleItem):
    implements(interfaces.IicSemanticThesaurusConfiguration)

    keywords_incrementalsearchwidget = ToolDependentFieldProperty(interfaces.IicSemanticThesaurusConfiguration['keywords_incrementalsearchwidget'])

def icSemanticThesaurusConfiguration_adapter(context):
    pcm = getUtility(interfaces.IicSemanticThesaurusConfiguration,
                     name='icsemantic.thesaurus.configuration',
                     context=context)
    app = context.restrictedTraverse('/')
    if exist_keywords_incrementalsearchwidget_property(app):
        pcm.keywords_incrementalsearchwidget = keywords_incrementalsearchwidget(app)
    return pcm



from AccessControl import ClassSecurityInfo
from Products.Archetypes.Widget import TypesWidget
from Products.CMFPlone.utils import safe_unicode
from icsemantic.thesaurus.Thesaurus import thesaurus_utility
from incrementalSearchSelectWidget import *
from Products.Archetypes.utils import unique
from pyThesaurus.Thesaurus import ConceptsConflict
import cgi

class IncrementalSearchSelectWidget(TypesWidget):
    _properties = TypesWidget._properties.copy()
    _properties.update({
        'size'  : 5,
        'roleBasedAdd' : True,
        })

    security = ClassSecurityInfo()

    def __init__(self, **kwargs):
        kwargs['macro'] = "incrementalSearchSelectArchetypesWidget"
        super(IncrementalSearchSelectWidget, self).__init__(**kwargs)

    def __call__(self, widgetName, mode='edit', arg=None):
        #html = self.template()
        self.set_name(widgetName)
        return super(IncrementalSearchSelectWidget, self).__call__(widgetName, mode, arg)

    security.declarePublic('process_form')    
    def process_form(self, instance, field, form, empty_marker=None,
                     emptyReturnsMarker=False):
        """process keywords from form where this widget has a list of
        available keywords and any new ones"""
        name = field.getName()
        new_keywords_string = form.get('%s' % name, empty_marker)
        if new_keywords_string is empty_marker:
            new_keywords = []
        else:
            new_keywords = self.terms_with_language(self.without_duplicated_and_empty(lstrip(new_keywords_string.split(","))))
            if self.allowToAddNewTermsToThesaurus():
                self.addNewTermsToThesaurus(new_keywords)

        value = new_keywords

        if not value and emptyReturnsMarker: return empty_marker

        return value, {}
    
    def without_duplicated_and_empty(self, aList):
        return dict([ (term,True) for term in aList if term ]).keys()
        
    def safe_name(self):
        return safe_name(self.name)
    
    def name(self):
        return _name
    
    def set_name(self, aName):
        _name = aName
    
    def getInputValue(self):
        return self.selectedConcepts()
    
    def selectedConcepts(self):
        return [ int(strcid) for strcid in self.request["%s_concepts" % safe_name(self.name)].split(",") ]
    
    def termsToConcepts(self, terms):
        return [ self.conceptWithPreferedTerm(term) for term in terms ]
    
    def terms_with_language(self, terms):
        return [ self.term_with_language(term) for term in terms ]
    
    def term_with_language(self, aTerm):
        if not "@" in aTerm:
            return aTerm + "@" + preferredLanguage()
        else:
            return aTerm
    
    def conceptWithPreferedTerm(self, aTerm):
        cids = [ cid for cid in self.conceptsWithTerm(aTerm) if aTerm in thesaurus_utility()[cid].get_prefered(supportedLanguages()) ]
        if len(cids) > 0:
            concept = str(cids[0])
        else:
            concept = aTerm
        return concept
    
    def conceptsWithTerm(self, aTerm):
        try:
            cids = thesaurus_utility()(aTerm)
        except IndexError:
            cids = []
        return cids
    
    def listToString(self, aList):
        if len(aList) == 0:
            aString = ""
        else:
            aString = ", ".join(aList) + ", "
        return aString
    
    def addNewTermsToThesaurus(self, new_terms):
        for term in self.termsToConcepts(new_terms):
            try:
                cid = int(term) #it is a concept
            except: #it is a term
                try:
                    thesaurus_utility().append_term(term)
                except ConceptsConflict, e:
                    print "Warning: The term " + str(e.value) + " already exist in the thesaurus."
    
    def allowToAddNewTermsToThesaurus(self):
        return True



from Products.CMFCore.utils import getToolByName
from browser.archetypes import IncrementalSearchSelectWidget
from Products.Archetypes.Widget import KeywordWidget

def replace_keywords_widget_at_metadata_page(out):
    print >> out, "Replacing keywords widget to incremental search widget at metadata page..."
    from Products.ATContentTypes.content.document import ATDocument
    ATDocument('widget-temp-document').schema['subject'].widget = IncrementalSearchSelectWidget(
                                                                label="Keywords",
                                                                label_msgid="label_keywords",
                                                                description_msgid="help_keyword",
                                                                i18n_domain="plone")
    print >> out, "Replaced."

def replace_keywords_widget_to_original_at_metadata_page(out):
    print >> out, "Replacing keywords widget to original one at metadata page..."
    from Products.ATContentTypes.content.document import ATDocument
    ATDocument('widget-temp-document').schema['subject'].widget = KeywordWidget(
                                                                    label="Keywords",
                                                                    label_msgid="label_keywords",
                                                                    description_msgid="help_keyword",
                                                                    i18n_domain="plone")
    print >> out, "Replaced."



from zope.app.form.browser.widget import SimpleInputWidget
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from Products.CMFPlone.utils import safe_unicode
from Products.CMFCore.utils import getToolByName
from icsemantic.thesaurus.Thesaurus import thesaurus_utility
from zope.app.component.hooks import getSite
import cgi

try:
    import Products.CMFPlone.migrations.v3_0
    HAS_PLONE3 = True
except ImportError:
    HAS_PLONE3 = False

try:
    from azax import AzaxBaseView, force_unicode
    from azax.plugins.core.interfaces import IKSSCoreCommands
    from azax.plugins.effects.interfaces import IScriptaculousEffectsCommands
except ImportError:
    try:
        from Products.azax import AzaxBaseView, force_unicode
        from Products.azax.plugins.core.interfaces import IKSSCoreCommands
        from Products.azax.plugins.effects.interfaces import IScriptaculousEffectsCommands
    except ImportError:
        from kss.core import force_unicode
        from kss.core.kssview import AzaxBaseView
        from kss.core.kssview import KSSView
        from kss.core.plugins.core.interfaces import IKSSCoreCommands
        from kss.core.plugins.effects.interfaces import IScriptaculousEffectsCommands

if not HAS_PLONE3:
    KSSView = AzaxBaseView

class IncrementalSearchSelectWidget(SimpleInputWidget):

    template = ViewPageTemplateFile('templates/incrementalSearchSelect-widget.pt')

    def __init__(self, context, request, thesaurus_context="", thesaurus_languages=[]):
        super(IncrementalSearchSelectWidget, self).__init__(context, request)
        self.extra = " autocomplete='off' concepts='' selectedIndex='' onfocus='timerIncrementalSearchSelect(this.id);' onkeypress='return keypressed(this.id, event)' size='50'"
        self._thesaurus_context = thesaurus_context
        self._thesaurus_languages = thesaurus_languages

    def __call__(self):
        html = self.template()
        html += super(IncrementalSearchSelectWidget, self).__call__()
        html += '<input type="hidden" name="%s_autocompletedExpression" id="%s_autocompletedExpression" value="" concepts="" selectedIndex="">' % (safe_name(self.name), safe_name(self.name))
        html += '<input type="hidden" name="%s_concepts" id="%s_concepts" value="">' % (safe_name(self.name), safe_name(self.name))
        html += '<input type="hidden" name="%s_selectedIndex" id="%s_selectedIndex" value="">' % (safe_name(self.name), safe_name(self.name))
        html += '<input type="hidden" name="%s_autocompleteConceptsAction" id="%s_autocompleteConceptsAction" class="incrementalSearchSelect" widgetName="%s" value="" concepts="">' % (self.name, self.name, self.name)
        html += '<input type="hidden" name="%s_chooseSelectedConceptAction" id="%s_chooseSelectedConceptAction" class="chooseSelectedConcept" widgetName="%s" value="" concepts="">' % (self.name, self.name, self.name)
        html += '<div id="div_%s"></div>' % safe_name(self.name) #TODO: replace to query
        return html
    
    def getInputValue(self):
        return self.selectedConcepts()
    
    def selectedConcepts(self):
        return [ int(strcid) for strcid in self.request["%s_concepts" % safe_name(self.name)].split(",") ]

class IncrementalSearchSelectAzax(KSSView):

    def autocompleteConcepts(self, widgetname, searchexpression, concepts, selectedindex):
        self.set_widgetName(widgetname)
        self.set_searchExpression(safe_unicode(searchexpression))
        self.set_concepts(safe_unicode(concepts))
        self.set_selectedIndex(selectedindex)
        self.detect_modifying_concept()
        self.html_showConceptsList()
        self.html_setSelectedIndex(str(self.selectedIndex()))
        return self.render()
    
    def detect_modifying_concept(self):
        terms = self.searchExpressionTerms()
        pref = self.selectedPreferedTerms()
        self.set_modifyingConcept(len(terms) - 1)
        for i in range(min(len(terms), len(pref))):
            if terms[i] != safe_unicode(pref[i]):
                self.set_modifyingConcept(i)
                break
    
    def html_showConceptsList(self):
        IKSSCoreCommands(self).replaceInnerHTML('div#div_%s' % self.safeWidgetName(), self.html_someConceptsList())

    def html_setSelectedIndex(self, aString):
        IKSSCoreCommands(self).setAttribute('input#%s_selectedIndex' % self.safeWidgetName(), "value", aString)

    def chooseSelectedConcept(self, widgetname, searchexpression, concepts, selectedindex):
        self.set_widgetName(widgetname)
        self.set_searchExpression(safe_unicode(searchexpression))
        self.set_concepts(safe_unicode(concepts))
        self.set_selectedIndex(selectedindex)
        self.detect_modifying_concept()
        (newSearchExpression, selectedConcept) = self.html_conceptSelectedSearchExpression()
        self.html_updateAutocompletedExpression(newSearchExpression)
        if selectedConcept != "" or self.allow_new_terms():
            self.updateSelectedConcept(selectedConcept)
            self.html_updateSelectedConcepts()
            self.html_hideConceptsList()
            self.html_setSelectedIndex("")
        return self.render()
    
    def updateSelectedConcept(self, selectedConcept):
        if self.modifyingConcept() > len(self.cids()) - 1:
            self.appendConcept(selectedConcept)
        else:
            self.updateConcept(self.modifyingConcept(), selectedConcept)

    def html_updateAutocompletedExpression(self, newSearchExpression):
        IKSSCoreCommands(self).setAttribute('input#%s_autocompletedExpression' % self.safeWidgetName(), "value", newSearchExpression)

    def html_hideConceptsList(self):
        IKSSCoreCommands(self).replaceInnerHTML('div#div_%s' % self.safeWidgetName(), "")

    def html_updateSelectedConcepts(self):
        IKSSCoreCommands(self).setAttribute('input#%s_autocompletedExpression' % self.safeWidgetName(), "concepts", self.concepts())

    def html_conceptSelectedSearchExpression(self):
        search_words = self.searchExpressionTerms()
        current_input = self.currentInput()
        if current_input == "":
            newSearchExpression = self.searchExpression()
            selectedConcept = ""
        else:
            results = self.query(current_input)
            if results['concepts_count'] > 0:
                selected_word = results['concepts'][self.selectedSubindex()][0]
                selectedConcept = results['concepts'][self.selectedSubindex()][1]
                search_words[self.modifyingConcept()] = selected_word
                newSearchExpression = (u", ").join(search_words)
                if self.modifyingConcept() == len(search_words) - 1:
                    newSearchExpression += (u", ")
            elif self.allow_new_terms(): #ACA SE CAMBIARAN LAS CONDICIONES
                if self.modifyingConcept() == len(search_words) - 1:
                    newSearchExpression = self.searchExpression() + (u", ")
                else:
                    newSearchExpression = self.searchExpression()
                selectedConcept = current_input
            else:
                newSearchExpression = self.searchExpression()
                selectedConcept = ""
        return (safe_unicode(newSearchExpression), selectedConcept)

    def html_someConceptsList(self):
        search_words = self.searchExpressionTerms()
        current_input = self.currentInput()
        html = ""
        if current_input != "":
            results = self.query(current_input)
            self.correct_selectedIndex(results['concepts_count'])
            if results['concepts_count'] > 0:
                selected_word = results['concepts'][self.selectedSubindex()][0]
            else:
                selected_word = ""

            html += "<table cellspacing='0' cellpadding='0' border='0'>"
            html += "    <tr>"
            html += "        <td><div id='searchResults' class='incrementalSearchSelectResults'>"
            for (term, cid) in results['concepts']:
                html += self.html_word_item(term, selected_word, current_input)
            if results['concepts_count'] > self.max_results():
                html += "            ...<br>"
            html += "            </div></td>"
            html += "        <td height='100%'>"
            if results['concepts_count'] > self.max_results():
                html += "<table cellspacing='0' cellpadding='0' border='0' height='100%'>"
                html += "                <tr><td style='vertical-align: top'><img src='arrowUp.gif' border='0' onclick='javascript:increment_selectedIndex(\"" + self.widgetName() + "\", -1);'></td></tr>"
                html += "                <tr><td style='vertical-align: bottom'><img src='arrowDown.gif' border='0' onclick='javascript:increment_selectedIndex(\"" + self.widgetName() + "\", +1);'></td></tr>"
                html += "            </table>"
            else:
                html += "&nbsp;"
            html += "</td>"
            html += "    </tr>"
            html += "</table>"
        return html

    def html_word_item(self, word, selected_word, current_input):
        if word == selected_word:
            return u"            <i>%s</i><br>" % self.html_emphasize_input(word, current_input)
        else:
            return u"            %s<br>" % self.html_emphasize_input(word, current_input)

    def html_emphasize_input(self, word, current_input):
        if current_input != "":
            return safe_unicode((u"<b>" + current_input + u"</b>").join(word.split(current_input)))
        else:
            return safe_unicode(word)
    
    def query(self, search_expression):
        return thesaurus_utility().query(search_expression=search_expression,
                                            narrowerthan = None,
                                            broaderthan = None,
                                            contexts = [],
                                            languages = supportedLanguages(),
                                            inbranch = None,
                                            hidden = None,
                                            max_results = self.max_results(),
                                            first_result = self.first_result())

    def first_result(self):
        return self.selectedIndex() - self.selectedSubindex()

    def selectedSubindex(self):
        return self.selectedIndex() % self.max_results()
    
    def max_results(self):
        return 5
    
    def allow_new_terms(self):
        return True

    def selectedIndex(self):
        return self._selectedIndex

    def set_selectedIndex(self, index):
        if index == "":
            self._selectedIndex = 0
        else:
            self._selectedIndex = int(index)

    def widgetName(self):
        return self._widgetName

    def set_widgetName(self, widgetName):
        self._widgetName = widgetName
    
    def safeWidgetName(self):
        return safe_name(self.widgetName())

    def correct_selectedIndex(self, concepts_count):
        self._selectedIndex = thesaurus_utility().correct_index(self._selectedIndex, concepts_count)

    def searchExpression(self):
        return self._searchExpression

    def set_searchExpression(self, aString):
        self._searchExpression = aString

    def concepts(self):
        return u",".join([ self.to_string(cid) for cid in self.cids() ])

    def set_concepts(self, aString):
        if aString == "":
            self.set_cids([])
        else:
            self.set_cids([self.to_int(str_cid) for str_cid in aString.split(",")])
    
    def appendConcept(self, cid):
        self.cids().append(cid)
    
    def updateConcept(self, index, cid):
        self.cids()[index] = cid
    
    def to_int(self, aString):
        try:
            aNumber = int(aString)
        except:
            aNumber = aString
        return aNumber
    
    def to_string(self, var):
        try:
            aString = str(var)
        except:
            aString = var
        return aString

    def cids(self):
        return self._cids

    def set_cids(self, aList):
        self._cids = aList

    def modifyingConcept(self):
        return self._modifyingConcept

    def set_modifyingConcept(self, aNumber):
        self._modifyingConcept = aNumber
    
    def searchExpressionTerms(self):
        return lstrip(self.searchExpression().split(","))
    
    def currentInput(self):
        return self.searchExpressionTerms()[self.modifyingConcept()]

    def selectedPreferedTerms(self):
        pref = []
        for cid in self.cids():
            try:
                pref.append(self.preferedTerm(int(cid)))
            except ValueError:
                pref.append(cid)
        return pref
    
    def preferedTerm(self, cid):
        return thesaurus_utility()[cid].get_prefered(supportedLanguages())[0]

def IncrementalSearchSelectWidgetFactory(context, request, thesaurus_context="", thesaurus_languages=[]):
    return IncrementalSearchSelectWidget(context, request, thesaurus_context, thesaurus_languages)

def safe_name(aString):
    return aString.replace(".", "")
        
def lstrip(words):
    return [ safe_unicode(word.lstrip()) for word in words ]
    
def preferredLanguage():
    return portal_languages().getPreferredLanguage()

def supportedLanguages():
    return portal_languages().getSupportedLanguages()

def portal_languages():
    return getToolByName(getSite(), 'portal_languages')



# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: __init__.py 358 2008-07-29 18:56:21Z flarumbe $
#
# end: Platecom header

from StringIO import StringIO
from Products.CMFCore.DirectoryView import registerDirectory
from zope.component import getUtility
from replaceKeywordsWidget import replace_keywords_widget_at_metadata_page

GLOBALS = globals()
registerDirectory('skins', GLOBALS)

def initialize(context):
    """Initializer called when used as a Zope 2 product."""

    # Apply monkey patches
    import patches
    
    if keywords_incrementalsearchwidget(context._ProductContext__app):
        replace_keywords_widget_at_metadata_page(StringIO())
    else:
        print "Warning: Keywords Incremental Search Widget is disabled."

def exist_keywords_incrementalsearchwidget_property(app):
    return 'icsemantic.thesaurus.configuration.keywords_incrementalsearchwidget' in app.propertyIds()

def keywords_incrementalsearchwidget(app):
    if exist_keywords_incrementalsearchwidget_property(app):
        return app['icsemantic.thesaurus.configuration.keywords_incrementalsearchwidget']
    else:
        return False

def add_keywords_incrementalsearchwidget_property_ifnotexists(app):
    if not exist_keywords_incrementalsearchwidget_property(app):
        app._setProperty('icsemantic.thesaurus.configuration.keywords_incrementalsearchwidget', True)

def update_keywords_incrementalsearchwidget_property(app, value):
    app._updateProperty('icsemantic.thesaurus.configuration.keywords_incrementalsearchwidget', value)
